package com.example.ellegadodepintia.exploradoresDePintia.model

import android.content.Intent
import android.os.Handler
import android.os.Looper
import android.widget.TextView
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.Resumen
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState.context
import com.example.ellegadodepintia.exploradoresDePintia.model.GestorCombate.cargarAnimacionesMuerteProtagonista
import com.example.ellegadodepintia.exploradoresDePintia.model.habilidad.Habilidad
import com.example.ellegadodepintia.exploradoresDePintia.model.habilidad.efectoHabilidad.EfectoGolpeMultiple
import com.example.ellegadodepintia.exploradoresDePintia.model.habilidad.efectoHabilidad.EfectoBuffo
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.Objeto
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.ObjetoEquipable
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.ObjetoUtilidad
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.efectoObjeto.EfectoEquipableAumento

class Jugador {
    var atributos: MutableMap<Atributo, Int> = mutableMapOf()
    var vidaMax = 10
    var energiaMax = 10
    private val observers = mutableListOf<(Jugador) -> Unit>()
    var inventario = ArrayList<Objeto>(9)
    var equipables = ArrayList<ObjetoEquipable>(3)
    var habilidades = ArrayList<Habilidad>(1)
    var buffos = ArrayList<EfectoBuffo>()
    var monedas = 0

    init {
        atributos[Atributo.Vida] = 10
        atributos[Atributo.Ataque] = 1
        atributos[Atributo.Investigacion] = 1
        atributos[Atributo.Energia] = 10

        inventario.add(
            ObjetoEquipable(
                nombre = "Espada de Hierro",
                descripcion = "Una espada de hierro sencilla y confiable perfecta para un aventurero novato.",
                efecto = EfectoEquipableAumento(1, Atributo.Ataque),
                imagen = R.drawable.asi_objeto_espadadefault
            )
        )

        inventario.add(
            ObjetoUtilidad(
                nombre = "Pico Vacceo",
                descripcion = "Un pico viejo y oscuro con dibujos extraños que parecen brillar cerca de las piedras, como si estuviera esperando que lo uses para algo importante.",
                imagen = R.drawable.asi_objeto_pico
            )
        )


        habilidades.add(
            Habilidad(
                nombre = "Furia del Vacceo",
                descripcion = "Libera todo el poder de su entrenamiento, entrando en un estado de furia que aumenta el daño en 2 puntos. Este efecto dura 2 turnos",
                efecto = EfectoBuffo(Atributo.Ataque, 2, 2),
                3
            )
        )

        habilidades.add(
            Habilidad(
                nombre = "Golpe Doble",
                descripcion = "Un ataque básico con el gladius que golpea dos veces con un 100% de precisión, causando el doble de daño al enemigo.",
                efecto = EfectoGolpeMultiple(2),
                3
            )
        )

        /*habilidades.add(
            Habilidad(
                nombre = "Revitalización Floral",
                descripcion = "Usando el poder ancestral de las plantas, esta habilidad invoca la energía curativa de las flores. " +
                        "Aumentas tu salud en 2 puntos",
                efecto = EfectoSubirAtaque(2)
            )
        )*/
    }

    fun addObserver(observer: (Jugador) -> Unit) {
        observers.add(observer)
    }

    fun removeObserver(observer: (Jugador) -> Unit) {
        observers.remove(observer)
    }

    private fun notifyObservers() {
        observers.forEach { it(this) }
    }

    fun actualizarAtributo(atributo: Atributo, valor: Int) {
        atributos[atributo] = atributos[atributo]!!.plus(valor)
        notifyObservers()
        if (atributo == Atributo.Vida && atributos[atributo]!! <= 0){
            "¡Has sido derrotado!".also { context!!.findViewById<TextView>(R.id.textoResultado).text = it }
            cargarAnimacionesMuerteProtagonista()
            Handler(Looper.getMainLooper()).postDelayed({
                val intent = Intent(context, Resumen::class.java)
                context!!.startActivity(intent)
            }, 2000)
        }
    }

    fun setHabilidad(atributo: Atributo, valor: Int) {
        atributos[atributo] = valor
        notifyObservers()
    }

    fun conseguirObjeto(objeto: Objeto): String {
        return if (inventario.size < 9) {
            inventario.add(objeto)
            "Se ha añadido ${objeto.nombre} al inventario"
        } else {
            "¡El inventario está lleno!"
        }
    }

    fun equiparObjeto(objeto: ObjetoEquipable) {
        equipables.add(objeto)
    }

    fun desequiparObjeto(objeto: ObjetoEquipable) {
        equipables.remove(objeto)
        inventario.add(objeto)
        notifyObservers()
    }

    fun destruirObjeto(objeto: Objeto) {
        inventario.remove(objeto)
        notifyObservers()
    }

    fun actualizarBuffo(efectoBuffo: EfectoBuffo) {
        buffos.add(efectoBuffo)
    }

    fun eliminarBuffo(efectoBuffo: EfectoBuffo) {
        buffos.remove(efectoBuffo)
    }

    fun eliminarTodosLosBuffos() {
        for (buffo in buffos) {
            buffo.eliminarBuffo()
        }

    }

    fun actualizarVidaMax(valor: Int) {
        vidaMax = valor
        notifyObservers()
    }

    fun actualizarMonedas(valor: Int){
        monedas = valor
        notifyObservers()
    }

    fun reset() {
        observers.clear()

        equipables.toList().forEach { it.desequipar() }

        inventario.clear()
        equipables.clear()
        buffos.clear()

        atributos[Atributo.Vida] = 10
        atributos[Atributo.Ataque] = 1
        atributos[Atributo.Investigacion] = 1
        atributos[Atributo.Energia] = 10

        vidaMax = 10
        energiaMax = 10
    }

}
