package com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory

import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.Opcion
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoDetallesCompra
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoIgnorar
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.ObjetoConsumible
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.ObjetoEquipable
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.efectoObjeto.EfectoEquipableAumento
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.efectoObjeto.EfectoObjetoCura



class TiendaFactory : EventoFactory {
    private val tiendasMap = mapOf(
        "Te encuentras al mercader de Pintia, quien te ofrece una pocíon que te restaura 3 puntos de vida por 5 monedas. ¿Te atreves a descubrir su poder?" to Pair(
            R.drawable.asi_exploradores_tienda, listOf(
                Opcion(
                    descripcion = "Ver Objeto",
                    atributoRequerida = Atributo.Nula,
                    EfectoDetallesCompra(
                        5,
                        ObjetoConsumible(nombre = "Elixir Menor de la Necrópolis",
                        descripcion = "Un frasco pequeño con un líquido granate y brillante, que destella con un resplandor profundo, capaz de restaurar 3 puntos de vida al instante.",
                        efecto = EfectoObjetoCura(3),
                        imagen = R.drawable.asi_objeto_pocima_vida_5
                    )
                    )
                ), Opcion(
                    descripcion = "Rechazar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
        "Te encuentras al mercader de Pintia, quien te ofrece una pocíon que te restaura 5 puntos de vida por 8 monedas. ¿Te atreves a probarla y sentir su poder renovador?" to Pair(
            R.drawable.asi_exploradores_tienda, listOf(
                Opcion(
                    descripcion = "Ver Objeto",
                    atributoRequerida = Atributo.Nula,
                    EfectoDetallesCompra(
                        8,
                        ObjetoConsumible(nombre = "Elixir Mediano de la Necrópolis",
                            descripcion = "Un frasco robusto con un líquido amarillo brillante, que resplandece como el sol al mediodía, capaz de restaurar 5 puntos de vida al instante.",
                            efecto = EfectoObjetoCura(5),
                            imagen = R.drawable.asi_objeto_pocima_vida_10
                        )
                    )
                ), Opcion(
                    descripcion = "Rechazar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
        "Te encuentras al mercader de Pintia, quien te ofrece una pocíon que te restaura toda la vida por 15 monedas. ¿Te atreves a probarla?" to Pair(
            R.drawable.asi_exploradores_tienda, listOf(
                Opcion(
                    descripcion = "Ver Objeto",
                    atributoRequerida = Atributo.Nula,
                    EfectoDetallesCompra(
                        15,
                        ObjetoConsumible(nombre = "Elixir Supremo de la Necrópolis",
                            descripcion = "Frasco supremo forjado con los secretos de los vacceos, restaura toda tu vitalidad al instante, ¡la poción de cura más poderosa de Pintia!",
                            efecto = EfectoObjetoCura(999),
                            imagen = R.drawable.asi_objeto_pocima_vida_all
                        )
                    )
                ), Opcion(
                    descripcion = "Rechazar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
        "Te encuentras al mercader de Pintia, quien te ofrece un escudo equipable que aumenta tu vida máxima en 10 puntos por 50 monedas. ¿Listo para lucirlo en combate?" to Pair(
            R.drawable.asi_exploradores_tienda, listOf(
                Opcion(
                    descripcion = "Ver Objeto",
                    atributoRequerida = Atributo.Nula,
                    EfectoDetallesCompra(
                        50,
                        ObjetoEquipable(nombre = "Escudo Caetrae",
                            descripcion = "La caetrae es un escudo redondo vacceo, forjado para la defensa en combate, que aumenta tu vida máxima en 10 puntos.",
                            efecto = EfectoEquipableAumento(10, Atributo.Vida),
                            imagen = R.drawable.asi_objeto_escudo
                        )
                    )
                ), Opcion(
                    descripcion = "Rechazar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
        "Te encuentras al mercader de Pintia, quien te ofrece un casco equipable que aumenta tu vida máxima en 8 puntos por 40 monedas. ¿Te atreves a equiparlo?" to Pair(
            R.drawable.asi_exploradores_tienda, listOf(
                Opcion(
                    descripcion = "Ver Objeto",
                    atributoRequerida = Atributo.Nula,
                    EfectoDetallesCompra(
                        40,
                        ObjetoEquipable(nombre = "Casco Ceremonial",
                            descripcion = "Casco ceremonial de oro de los vacceos decorado con intrincados patrones que simboliza el estatus social. Aumenta tu vida máxima en 8 puntos ",
                            efecto = EfectoEquipableAumento(8, Atributo.Vida),
                            imagen = R.drawable.asi_objeto_casco
                        )
                    )
                ), Opcion(
                    descripcion = "Rechazar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),

        )

    override fun generarDescripcion(): String {
        return tiendasMap.keys.random()
    }

    override fun generarOpciones(descripcion: String): MutableList<Opcion> {
        return tiendasMap[descripcion]?.second?.toMutableList()
            ?: mutableListOf()
    }

    override fun generarImagen(descripcion: String): Int {
        return tiendasMap[descripcion]?.first!!
    }

    override fun generarEvento(): EventoTienda {
        val descripcion = generarDescripcion()
        return EventoTienda(
            descripcion = descripcion,
            dificultad = 0,
            opciones = generarOpciones(descripcion),
            imagen = generarImagen(descripcion),
            resistenciaMax = (2..8).random(),
            finalizado = false
        )
    }
}